@@ -58,7 +58,7 @@ class SijaxMainTestCase(unittest.TestCase):
         inst.set_data({"key": "value"})
         self.assertFalse(inst.is_sijax_request)
         self.assertTrue(inst.requested_function is None)
-        
+
         # missing args
         inst.set_data({cls.PARAM_REQUEST: "function"})
         self.assertFalse(inst.is_sijax_request)
@@ -103,9 +103,9 @@ class SijaxMainTestCase(unittest.TestCase):
         self.assertFalse(inst.has_event(event_name), "Custom event registered")
 
         inst.register_event(event_name, event_callback)
-        
+
         self.assertTrue(inst.has_event(event_name), "Failed to register event")
-                
+
     def test_executing_regular_callbacks_works(self):
         from types import StringType
 
@@ -245,7 +245,7 @@ class SijaxMainTestCase(unittest.TestCase):
 
         def my_callback(obj_response, arg1, arg2):
             call_history.append("call ok")
-        
+
         def my_callback_with_defaults(obj_response, arg1=138, arg2=15):
             call_history.append("defaults ok")
 
@@ -263,9 +263,9 @@ class SijaxMainTestCase(unittest.TestCase):
 
         inst = Sijax()
         cls = inst.__class__
-        inst.register_event(cls.EVENT_INVALID_CALL, invalid_call)
         inst.register_callback("my_func", my_callback)
 
+        # Make a valid call that would succeed
         inst.set_data({cls.PARAM_REQUEST: "my_func", cls.PARAM_ARGS: '["arg1", 12]'})
         self.assertTrue(inst.is_sijax_request)
         self.assertEqual("my_func", inst.requested_function)
@@ -273,8 +273,29 @@ class SijaxMainTestCase(unittest.TestCase):
         response = inst.process_request()
         self.assertTrue(isinstance(response, StringType))
 
-        # we should have succeeded until now..
-        # let's try to make the call invalid and observe the failure
+
+        # Make a call with a wrong number of arguments, and a default
+        # event handler for invalid calls
+        inst.set_data({cls.PARAM_REQUEST: "my_func", cls.PARAM_ARGS: '["arg1"]'})
+        self.assertTrue(inst.is_sijax_request)
+        self.assertEqual("my_func", inst.requested_function)
+        self.assertEqual(["arg1"], inst.request_args)
+        response = inst.process_request()
+        self.assertTrue(isinstance(response, StringType))
+        try:
+            commands = json.loads(response)
+        except:
+            self.fail("Invalid JSON generated!")
+        else:
+            self.assertTrue(isinstance(commands, list))
+            # we expect the default "Action performed in a wrong way" alert
+            self.assertEqual(1, len(commands))
+            command_data = commands.pop(0)
+            self.assertTrue("type" in command_data)
+            self.assertEqual("alert", command_data["type"])
+
+        # Make an invalid call with a custom event handler function
+        inst.register_event(cls.EVENT_INVALID_CALL, invalid_call)
         inst.set_data({cls.PARAM_REQUEST: "my_func", cls.PARAM_ARGS: '[]'})
         self.assertEqual("my_func", inst.requested_function)
         self.assertEqual([], inst.request_args)
@@ -427,7 +448,7 @@ class SijaxMainTestCase(unittest.TestCase):
             call_history.append(arg1_custom)
             call_history.append(arg2_custom)
             call_history.append(arg_regular)
-            
+
         def callback_basic(obj_response, arg_regular):
             self.assertTrue(isinstance(obj_response, BaseResponse))
             call_history.append(arg_regular)
@@ -447,7 +468,7 @@ class SijaxMainTestCase(unittest.TestCase):
 
         call_history_expected = ["one", "two", "regular", "reg2"]
         self.assertEqual(call_history_expected, call_history)
-    
+
     def test_args_extra_expects_a_list_or_a_tuple(self):
         inst = Sijax()
 
@@ -460,7 +481,7 @@ class SijaxMainTestCase(unittest.TestCase):
                 pass
 
             self.assertEqual(should_succeed, success, "Failure for %s" % repr(args_extra))
-                
+
         # A list, a tuple and None should all work
         try_args([], True)
         try_args((), True)
@@ -477,7 +498,7 @@ class SijaxMainTestCase(unittest.TestCase):
         try_args(False, False)
         try_args({"dictionary": "here"}, False)
 
-    def test_regula_functions_that_yield_are_not_allowed(self):
+    def test_regular_functions_that_yield_are_not_allowed(self):
         # Yielding is only supported by streaming functions.
         # It makes no sense for regular functions to use it.
         # If a regular function tries to yield, we expect
@@ -605,6 +626,39 @@ class SijaxMainTestCase(unittest.TestCase):
             fp.close()
             try_init(static_path, True)
 
+    def test_response_classes_need_to_be_callable(self):
+        # This ensures that register_callback will catch
+        # a custom response_class which is not really a class
+        # or a callable function which could produce a class instance
+
+        inst = Sijax()
+
+        def try_response_class(response_class, should_succeed):
+            success = False
+            try:
+                inst.register_callback("name", lambda r: r, response_class=response_class)
+                success = True
+            except SijaxError:
+                pass
+
+            self.assertEqual(should_succeed, success, "Failure for %s" % repr(response_class))
+
+        try_response_class(BaseResponse, True)
+
+        class CustomResponse(BaseResponse): pass
+        try_response_class(CustomResponse, True)
+
+        # A callable instead of a response class still works
+        def response_factory(*args, **kwargs):
+            return CustomResponse(*args, **kwargs)
+        try_response_class(response_factory, True)
+
+        # This works too, because None makes it use the default (BaseResponse)
+        try_response_class(None, True)
+
+        try_response_class('', False)
+        try_response_class(14, False)
+
 
 class SijaxStreamingTestCase(unittest.TestCase):
     """This tests the StreamingIframeResponse functionality, which is
@@ -660,7 +714,7 @@ class SijaxStreamingTestCase(unittest.TestCase):
         inst.register_event(cls.EVENT_BEFORE_PROCESSING, callback_before)
         inst.register_event(cls.EVENT_AFTER_PROCESSING, callback_after)
         options = {cls.PARAM_RESPONSE_CLASS: StreamingIframeResponse}
-        
+
         response = inst.execute_callback([], callback=callback_yielding, **options)
         self.assertTrue(isinstance(response, GeneratorType))
         # We should have the following yields:
@@ -679,7 +733,7 @@ class SijaxStreamingTestCase(unittest.TestCase):
             # flushing would be wasteful
             for i in range(10):
                 yield obj_response
-            
+
             for i in range(2):
                 obj_response.alert("hey")
                 yield obj_response
@@ -932,7 +986,7 @@ def suite():
     suite.addTest(unittest.makeSuite(SijaxStreamingTestCase))
     suite.addTest(unittest.makeSuite(SijaxCometTestCase))
     suite.addTest(unittest.makeSuite(SijaxUploadTestCase))
-    
+
     return suite
 
 
